# -*- coding: utf-8 -*-
from xml.etree.ElementTree import Element, SubElement

from numpy import (
    abs, asanyarray, cumsum, dtype, int8, int16, float16, float32, max,
    min_scalar_type)


def generate_polydata(
        points, vertices=None, lines=None, polygons=None, triangle_strips=None,
        point_data=None, cell_data=None):
    points = asanyarray(points)

    if vertices is not None:
        raise NotImplementedError('Adding vertices is not implemented yet.')

    if triangle_strips is not None:
        raise NotImplementedError(
            'Adding triangle strips is not implemented yet.')

    if cell_data is not None:
        raise NotImplementedError('Adding cell_data is not implemented yet.')

    if vertices is None:
        vertices = []

    if lines is None:
        lines = []

    if polygons is None:
        polygons = []

    if triangle_strips is None:
        triangle_strips = []

    if point_data is None:
        point_data = {}

    if cell_data is None:
        cell_data = {}

    root = Element(
        'VTKFile',
        attrib={
            'type': 'PolyData',
            'version': '0.1',
            'byte_order': 'LittleEndian'
        })
    polydata = SubElement(
        root,
        'PolyData')
    piece = SubElement(
        polydata,
        'Piece',
        attrib={
            'NumberOfPoints': str(len(points)),
            'NumberOfVerts': str(len(vertices)),
            'NumberOfLines': str(len(lines)),
            'NumberOfStrips': str(len(triangle_strips)),
            'NumberOfPolys': str(len(polygons))
        })

    # Points
    # ======
    points_dtype = min_scalar_type(-max(abs(points))-1)
    # Some datatypes are not supported by vtk and are hence mapped to the
    # closest supported datatype
    if points_dtype == int8:
        points_dtype = dtype(int16)
    elif points_dtype == float16:
        points_dtype = dtype(float32)

    points_element = SubElement(piece, 'Points')
    points_data_array = SubElement(
        points_element, 'DataArray',
        {'NumberOfComponents': str(points.shape[1]),
         'format': 'ascii',
         'type': points_dtype.name.capitalize()})
    points_data_array.text = ' '.join(
        ' '.join(str(coord) for coord in point) for point in points)

    # Lines
    # =====
    if lines:
        lines_element = SubElement(piece, 'Lines')

        # Connectivity
        lines_connectivity = SubElement(
            lines_element,
            'DataArray',
            {
                'Name': 'connectivity',
                'format': 'ascii',
                'type': 'Int32'
            })
        lines_connectivity.text = ' '.join(
            ' '.join(str(i) for i in connectivity) for connectivity in lines)

        # Offsets
        lines_offsets = SubElement(
            lines_element,
            'DataArray',
            {
                'Name': 'offsets',
                'format': 'ascii',
                'type': 'Int32'
            })
        lines_offsets.text = ' '.join(
            str(i) for i in cumsum([len(conn) for conn in lines]))

    # Polygons
    # ========
    if polygons:
        polygons_element = SubElement(piece, 'Polys')

        # Connectivity
        polygons_connectivity = SubElement(
            polygons_element,
            'DataArray',
            {
                'Name': 'connectivity',
                'format': 'ascii',
                'type': 'Int32'
            })
        polygons_connectivity.text = ' '.join(
            ' '.join(
                str(i) for i in connectivity) for connectivity in polygons)

        # Offsets
        polygons_offsets = SubElement(
            polygons_element,
            'DataArray',
            {
                'Name': 'offsets',
                'format': 'ascii',
                'type': 'Int32'
            })
        polygons_offsets.text = ' '.join(
            str(i) for i in cumsum([len(conn) for conn in polygons]))

    # Point Data
    # ==========
    if point_data:
        p_data = SubElement(piece, 'PointData')
        for name, data in point_data.items():
            data_array = SubElement(
                p_data,
                'DataArray',
                attrib={
                    'type': data.dtype.name.capitalize(),
                    'Name': name,
                    'format': 'ascii'
                }
            )

            data_array.text = ' '.join(str(i) for i in data)

    # Cell Data
    # =========
    if cell_data:
        c_data = SubElement(piece, 'CellData')
        for name, data in cell_data.items():
            data_array = SubElement(
                c_data,
                'DataArray',
                attrib={
                    'type': data.dtype.name.capitalize(),
                    'Name': name,
                    'format': 'ascii'
                }
            )

            data_array.text = ' '.join(str(i) for i in data)

    return root



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages
import ast
import re

root_dir = os.path.abspath(os.path.join(os.path.dirname(__file__)))

_version_re = re.compile(r'__version__\s+=\s+(.*)')
with open(os.path.join(root_dir, 'pysimplevtk', '__init__.py'), 'r') as f:
    version = str(ast.literal_eval(_version_re.search(f.read()).group(1)))

setup(
    name='pysimplevtk',
    version=version,
    packages=find_packages(),
    url='https://github.com/ZeeD26/pysimplevtk',
    license='BSD',
    author='Dominik Steinberger',
    author_email='zeed.dev@icloud.com',
    description='High level wrapper to work with vtk xml files.',
    install_requires=[
        'numpy'
    ])



# -*- coding: utf-8 -*-
from xml.etree.ElementTree import Element, SubElement

import numpy as np


def array_to_rectilinear_grid(coordinates, cell_data=None):
    if cell_data is None:
        cell_data = {}

    try:
        x = np.asanyarray(coordinates[0])
    except IndexError:
        x = np.array([0])

    try:
        y = np.asanyarray(coordinates[1])
    except IndexError:
        y = np.array([0])

    try:
        z = np.asanyarray(coordinates[2])
    except IndexError:
        z = np.array([0])

    root = Element(
        'VTKFile',
        attrib={
            'type': 'RectilinearGrid',
            'version': '0.1',
            'byte_order': 'LittleEndian'
        })
    rectilinear_grid = SubElement(
        root,
        'RectilinearGrid',
        attrib={
            'WholeExtent': ' '.join(
                str(i) for i in [0, x.size-1, 0, y.size-1, 0, z.size-1])
        })
    piece = SubElement(
        rectilinear_grid,
        'Piece',
        attrib={
            'Extent': ' '.join(
                str(i) for i in [0, x.size-1, 0, y.size-1, 0, z.size-1])
        })

    # Coordinates
    coordinates = SubElement(piece, 'Coordinates')
    for c in [x, y, z]:
        coords = SubElement(
            coordinates,
            'DataArray',
            attrib={
                'type': c.dtype.name.capitalize(),
                'format': 'ascii'
            })
        if c.size > 1:
            coords.set('RangeMin', str(c[0]))
            coords.set('RangeMax', str(c[-1]))
        coords.text = ' '.join(str(i) for i in c)

    # Cell data
    c_data = SubElement(piece, 'CellData')
    for name, data in cell_data.items():
        # VTK works with the fortran layout
        # print(data.flags['F_CONTIGUOUS'])
        # d = np.asfortranarray(data)
        # print(d.flags['F_CONTIGUOUS'])

        data_array = SubElement(
            c_data,
            'DataArray',
            attrib={
                'type': data.dtype.name.capitalize(),
                'Name': name,
                'format': 'ascii'
            }
        )

        data_array.text = ' '.join(str(i) for i in data.ravel(order='F'))

    return root
